package ly.count.android.sdk;

import androidx.test.ext.junit.runners.AndroidJUnit4;
import java.util.HashMap;
import java.util.Map;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;

import static androidx.test.InstrumentationRegistry.getContext;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.ArgumentMatchers.isNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;

@RunWith(AndroidJUnit4.class)
public class ModuleCrashTests {
    Countly mCountly;
    CountlyConfig config;
    ConnectionQueue connectionQueue;

    @Before
    public void setUp() {
        final CountlyStore countlyStore = new CountlyStore(getContext(), mock(ModuleLog.class));
        countlyStore.clear();

        mCountly = new Countly();
        config = (new CountlyConfig(getContext(), "appkey", "http://test.count.ly")).setDeviceId("1234").setLoggingEnabled(true).enableCrashReporting();
        mCountly.init(config);

        connectionQueue = mock(ConnectionQueue.class);
        mCountly.setConnectionQueue(connectionQueue);
    }

    @After
    public void tearDown() {
    }

    @Test
    public void setCrashFilters() {
        CrashFilterCallback callback = new CrashFilterCallback() {
            @Override
            public boolean filterCrash(String crash) {
                if (crash.contains("Secret")) {
                    return true;
                }
                return false;
            }
        };

        Countly countly = new Countly();
        CountlyConfig cConfig = (new CountlyConfig(getContext(), "appkey", "http://test.count.ly")).setDeviceId("1234").setLoggingEnabled(true).enableCrashReporting();
        cConfig.setCrashFilterCallback(callback);

        countly.init(cConfig);

        Assert.assertEquals(callback, countly.moduleCrash.crashFilterCallback);
    }

    @Test
    public void crashFilterTest() {
        Countly countly = new Countly();
        CountlyConfig cConfig = (new CountlyConfig(getContext(), "appkey", "http://test.count.ly")).setDeviceId("1234").setLoggingEnabled(true).enableCrashReporting();
        cConfig.setCrashFilterCallback(new CrashFilterCallback() {
            @Override
            public boolean filterCrash(String crash) {
                if (crash.contains("Secret")) {
                    return true;
                }
                return false;
            }
        });

        countly.init(cConfig);
        countly.setConnectionQueue(connectionQueue);

        Exception exception = new Exception("Secret message");

        countly.crashes().recordHandledException(exception);

        verify(connectionQueue, never()).sendCrashReport(any(String.class), any(boolean.class), any(boolean.class), isNull(Map.class));

        Throwable throwable = new Throwable("Secret message");

        countly.crashes().recordUnhandledException(throwable);

        verify(connectionQueue, never()).sendCrashReport(any(String.class), any(boolean.class), any(boolean.class), isNull(Map.class));

        exception = new Exception("Reasonable message");

        countly.crashes().recordHandledException(exception);

        ArgumentCaptor<String> arg = ArgumentCaptor.forClass(String.class);
        verify(connectionQueue).sendCrashReport(arg.capture(), eq(true), eq(false), isNull(Map.class));

        Assert.assertTrue(arg.getValue().startsWith("java.lang.Exception: Reasonable message\n" +
            "\tat ly.count.android.sdk.ModuleCrashTests.crashFilterTest(ModuleCrashTests.java:"));
    }

    @Test
    public void setCustomCrashSegment() {
        CrashDetails.customSegments = null;
        Countly countly = new Countly();
        CountlyConfig cConfig = (new CountlyConfig(getContext(), "appkey", "http://test.count.ly")).setDeviceId("1234").setLoggingEnabled(true).enableCrashReporting();

        Map<String, Object> segmIn = CrashDetails.customSegments;
        Assert.assertTrue(segmIn == null || segmIn.size() == 0);

        Map<String, Object> segm = new HashMap<>();
        segm.put("aa", "dd");
        segm.put("aa1", "dda");
        segm.put("1", 1234);
        segm.put("2", 1234.55d);
        segm.put("3", true);
        segm.put("4", 45.4f);
        segm.put("41", new Object());
        segm.put("42", new int[] { 1, 2 });

        cConfig.setCustomCrashSegment(segm);

        countly.init(cConfig);

        Map<String, Object> segm2 = new HashMap<>();
        segm2.put("aa", "dd");
        segm2.put("aa1", "dda");
        segm2.put("1", 1234);
        segm2.put("2", 1234.55d);
        segm2.put("3", true);

        Assert.assertEquals(segm2, CrashDetails.customSegments);
    }

    @Test
    public void addCrashBreadcrumb() {
        mCountly.crashes().addCrashBreadcrumb("Breadcrumb_1");
        mCountly.crashes().addCrashBreadcrumb("Breadcrumb_2");
        mCountly.crashes().addCrashBreadcrumb("Breadcrumb_3");

        String logs = CrashDetails.getLogs();

        Assert.assertEquals("Breadcrumb_1\nBreadcrumb_2\nBreadcrumb_3\n", logs);
    }

    @Test
    public void recordHandledExceptionException() {
        Exception exception = new Exception("Some message");

        mCountly.crashes().recordHandledException(exception);

        ArgumentCaptor<String> arg = ArgumentCaptor.forClass(String.class);
        verify(connectionQueue).sendCrashReport(arg.capture(), eq(true), eq(false), isNull(Map.class));

        Assert.assertTrue(arg.getValue().startsWith("java.lang.Exception: Some message\n" +
            "\tat ly.count.android.sdk.ModuleCrashTests.recordHandledExceptionException(ModuleCrashTests.java:"));
    }

    @Test
    public void recordHandledExceptionThrowable() {
        Throwable throwable = new Throwable("Some message");

        mCountly.crashes().recordHandledException(throwable);

        ArgumentCaptor<String> arg = ArgumentCaptor.forClass(String.class);
        verify(connectionQueue).sendCrashReport(arg.capture(), eq(true), eq(false), isNull(Map.class));

        String crash = arg.getValue();

        Assert.assertTrue(crash.startsWith("java.lang.Throwable: Some message\n" +
            "\tat ly.count.android.sdk.ModuleCrashTests.recordHandledExceptionThrowable(ModuleCrashTests.java:"));
    }

    @Test
    public void recordUnhandledExceptionException() {
        Exception exception = new Exception("Some message");

        mCountly.crashes().recordUnhandledException(exception);

        ArgumentCaptor<String> arg = ArgumentCaptor.forClass(String.class);
        verify(connectionQueue).sendCrashReport(arg.capture(), eq(false), eq(false), isNull(Map.class));

        String crash = arg.getValue();

        Assert.assertTrue(crash.startsWith("java.lang.Exception: Some message\n" +
            "\tat ly.count.android.sdk.ModuleCrashTests.recordUnhandledExceptionException(ModuleCrashTests.java:"));
    }

    @Test
    public void recordUnhandledExceptionThrowable() {
        Throwable throwable = new Throwable("Some message");

        mCountly.crashes().recordUnhandledException(throwable);

        ArgumentCaptor<String> arg = ArgumentCaptor.forClass(String.class);
        verify(connectionQueue).sendCrashReport(arg.capture(), eq(false), eq(false), isNull(Map.class));

        String crash = arg.getValue();

        Assert.assertTrue(crash.startsWith("java.lang.Throwable: Some message\n" +
            "\tat ly.count.android.sdk.ModuleCrashTests.recordUnhandledExceptionThrowable(ModuleCrashTests.java:"));
    }

    @Test(expected = StackOverflowError.class)
    public void crashTest_1() {
        mCountly.moduleCrash.crashTest(1);
    }

    @Test(expected = ArithmeticException.class)
    public void crashTest_2() {
        mCountly.moduleCrash.crashTest(2);
    }

    @Test(expected = RuntimeException.class)
    public void crashTest_4() {
        mCountly.moduleCrash.crashTest(3);
    }

    @Test(expected = NullPointerException.class)
    public void crashTest_5() {
        mCountly.moduleCrash.crashTest(4);
    }
}